# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 10 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
import os
import pandas as pd

from actsnclass.classifiers import random_forest
from actsnclass.query_strategies import uncertainty_sampling, random_sampling
from actsnclass.metrics import get_snpcc_metric


__all__ = ['DataBase']


class DataBase:
    """DataBase object, upon which the active learning loop is performed.

    Attributes
    ----------
    classprob: np.array
        Classification probability for all objects, [pIa, pnon-Ia].
    data: pd.DataFrame
        Complete information read from features files.
    features: pd.DataFrame
        Feature matrix to be used in classification (no metadata).
    features_names: list
        Header for attribute `features`.
    metadata: pd.DataFrame
        Features matrix which will not be used in classification.
    metadata_names: list
        Header for metadata.
    metrics_list_names: list
        Values for metric elements.
    predicted_class: np.array
        Predicted classes - results from ML classifier.
    queried_sample: list
        Complete information of queried objects.
    queryable_ids: np.array()
        Flag for objects available to be queried.
    test_features: pd.DataFrame
        Features matrix for the test sample.
    test_metadata: pd.DataFrame
        Metadata for the test sample
    test_labels: np.array()
        True classification for the test sample.
    train_features: pd.DataFrame
        Features matrix for the train sample.
    train_metadata: pd.DataFrame
        Metadata for the training sample.
    train_labels: np.array
        Classes for the training sample.

    Methods
    -------
    load_bazin_features(path_to_bazin_file: str)
        Load Bazin features from file
    load_features(path_to_file: str, method: str)
        Load features according to the chosen feature extraction method.
    build_samples(initial_training: str or int, nclass: int)
        Separate train and test samples.
    classify(method: str)
        Apply a machine learning classifier.
    evaluate_classification(metric_label: str)
        Evaluate results from classification.
    make_query(strategy: str, batch: int) -> list
        Identify new object to be added to the training sample.
    update_samples(query_indx: list)
        Add the queried obj(s) to training and remove them from test.
    save_metrics(loop: int, output_metrics_file: str)
        Save current metrics to file.
    save_queried_sample(queried_sample_file: str, loop: int, full_sample: str)
        Save queried sample to file.

    Examples
    --------
    >>> from actsnclass import DataBase

    Define the necessary paths

    >>> path_to_bazin_file = 'results/Bazin.dat'
    >>> metrics_file = 'results/metrics.dat'
    >>> query_file = 'results/query_file.dat'

    Initiate the DataBase object and load the data.
    >>> data = DataBase()
    >>> data.load_features(path_to_bazin_file, method='Bazin')

    Separate training and test samples and classify

    >>> data.build_samples(initial_training='original', nclass=2)
    >>> data.classify(method='RandomForest')
    >>> print(data.classprob)          # check predicted probabilities
    [[0.461 0.539]
    [0.346print(data.metrics_list_names)           # check metric header
    ['acc', 'eff', 'pur', 'fom']

    >>> print(data.metrics_list_values)          # check metric values
    [0.5975434599574068, 0.9024767801857585,
    0.34684684684684686, 0.13572404702012383] 0.654]
    ...
    [0.398 0.602]
    [0.396 0.604]]

    Calculate classification metrics

    >>> data.evaluate_classification(metric_label='snpcc')
    >>>

    Make query, choose object and update samples

    >>> indx = data.make_query(strategy='UncSampling', batch=1)
    >>> data.update_samples(indx)

    Save results to file

    >>> data.save_metrics(loop=0, output_metrics_file=metrics_file)
    >>> data.save_queried_sample(loop=0, queried_sample_file=query_file,
    >>>                          full_sample=False)
    """

    def __init__(self):
        self.classprob = np.array([])
        self.data = pd.DataFrame()
        self.features = pd.DataFrame([])
        self.features_names = []
        self.metadata = pd.DataFrame()
        self.metadata_names = []
        self.metrics_list_names = []
        self.metrics_list_values = []
        self.predicted_class = np.array([])
        self.queried_sample = []
        self.queryable_ids = np.array([])
        self.test_features = np.array([])
        self.test_metadata = pd.DataFrame()
        self.test_labels = np.array([])
        self.train_features = np.array([])
        self.train_metadata = pd.DataFrame()
        self.train_labels = np.array([])

    def load_bazin_features(self, path_to_bazin_file: str, screen=False):
        """Load Bazin features from file.

        Populate properties: data, features, feature_list, header
        and header_list.

        Parameters
        ----------
        path_to_bazin_file: str
            Complete path to Bazin features file.
        screen: bool (optional)
            If True, print on screen number of light curves processed.
            Default is False.
        """

        # read matrix with Bazin features
        self.data = pd.read_csv(path_to_bazin_file, sep=' ', index_col=False)

        # list of features to use
        self.features_names = ['gA', 'gB', 'gt0', 'gtfall', 'gtrise', 'rA',
                               'rB', 'rt0', 'rtfall', 'rtrise', 'iA', 'iB',
                               'it0', 'itfall', 'itrise', 'zA', 'zB', 'zt0',
                               'ztfall', 'ztrise']

        self.metadata_names = ['id', 'redshift', 'type', 'code', 'sample']

        self.features = self.data[self.features_names]
        self.metadata = self.data[self.metadata_names]

        if screen:
            print('Loaded ', self.metadata.shape[0], ' samples!')

    def load_features(self, path_to_file: str, method='Bazin',
                      screen=False):
        """Load features according to the chosen feature extraction method.

        Populates properties: data, features, feature_list, header
        and header_list.

        Parameters
        ----------
        path_to_file: str
            Complete path to features file.
        method: str (optional)
            Feature extraction method. The current implementation only
            accepts method=='Bazin'
        screen: bool (optional)
            If True, print on screen number of light curves processed.
            Default is False.
        """

        if method == 'Bazin':
            self.load_bazin_features(path_to_file, screen=screen)
        else:
            raise ValueError('Only Bazin features are implemented! '
                             '\n Feel free to add other options.')

    def build_samples(self, initial_training='original', nclass=2,
                      screen=False):
        """Separate train and test samples.

        Populate properties: train_features, train_header, test_features,
        test_header, queryable_ids (if flag available), train_labels and
        test_labels.

        Parameters
        ----------
        initial_training : str or int
            Choice of initial training sample.
            If 'original': begin from the train sample flagged in the file
            If int: choose the required number of samples at random,
            ensuring that at least half are SN Ia.
        nclass: int (optional)
            Number of classes to consider in the classification
            Currently only nclass == 2 is implemented.
        screen: bool (optional)
            If True display the dimensions of training and test samples.
        """

        # separate original training and test samples
        if initial_training == 'original':
            train_flag = self.metadata['sample'] == 'train'
            train_data = self.features[train_flag]
            self.train_features = train_data.values
            self.train_metadata = self.metadata[train_flag]

            test_flag = np.logical_or(self.metadata['sample'] == 'test',
                                      self.metadata['sample'] == 'queryable')
            test_data = self.features[test_flag]
            self.test_features = test_data.values
            self.test_metadata = self.metadata[test_flag]

            if 'queryable' in self.metadata['sample'].values:
                queryable_flag = self.metadata['sample'].values == 'queryable'
                self.queryable_ids = self.metadata[queryable_flag]['id'].values
            else:
                self.queryable_ids = self.test_metadata['id'].values

            if nclass == 2:
                train_ia_flag = self.train_metadata['type'] == 'Ia'
                self.train_labels = np.array([int(item) for item in train_ia_flag])

                test_ia_flag = self.test_metadata['type'] == 'Ia'
                self.test_labels = np.array([int(item) for item in test_ia_flag])
            else:
                raise ValueError("Only 'Ia x non-Ia' are implemented! "
                                 "\n Feel free to add other options.")

        elif isinstance(initial_training, int):

            # initialize the temporary label holder
            train_indexes = np.array([1])
            temp_labels = np.array([self.metadata['type'].values[train_indexes]])

            # make sure half are Ias and half are non-Ias
            while sum(temp_labels == 'Ia') != initial_training // 2 + 1:
                # this is an array of 5 indexes
                train_indexes = np.random.randint(low=0,
                                                  high=self.metadata.shape[0],
                                                  size=initial_training)
                temp_labels = self.metadata['type'].values[train_indexes]

            # set training
            train_flag = self.metadata['type'].values[train_indexes] == 'Ia'
            self.train_labels = np.array([int(item) for item in train_flag])
            self.train_features = self.features.values[train_indexes]
            self.train_metadata = pd.DataFrame(self.metadata.values[train_indexes],
                                               columns=self.metadata_names)

            # set test set as all objs apart from those in training
            test_indexes = np.array([i for i in range(self.metadata.shape[0])
                                     if i not in train_indexes])
            test_ia_flag = self.metadata['type'].values[test_indexes] == 'Ia'
            self.test_labels = np.array([int(item) for item in test_ia_flag])
            self.test_features = self.features.values[test_indexes]
            self.test_metadata = pd.DataFrame(self.metadata.values[test_indexes],
                                              columns=self.metadata_names)

            if 'queryable' in self.metadata['sample'].values:
                test_flag = np.array([True if i in test_indexes else False
                                      for i in range(self.metadata.shape[0])])
                queryable_flag = self.metadata['sample'] == 'queryable'
                combined_flag = np.logical_and(test_flag, queryable_flag)
                self.queryable_ids = self.metadata[combined_flag]['id'].values
            else:
                self.queryable_ids = self.test_metadata['id'].values

        else:
            raise ValueError('"Initial training" should be '
                             '"original" or integer.')

        if screen:
            print('Training set size: ', self.train_metadata.shape[0])
            print('Test set size: ', self.test_metadata.shape[0])

    def classify(self, method='RandomForest'):
        """Apply a machine learning classifier.

        Populate properties: predicted_class and class_prob

        Parameters
        ----------
        method: str (optional)
            Chosen classifier.
            The current implementation on accepts `RandomForest`.
        """

        if method == 'RandomForest':
            self.predicted_class,  self.classprob = \
                random_forest(self.train_features, self.train_labels,
                              self.test_features)

        else:
            raise ValueError('Only RandomForest classifier is implemented!'
                             '\n Feel free to add other options.')

    def evaluate_classification(self, metric_label='snpcc'):
        """Evaluate results from classification.

        Populate properties: metric_list_names and metrics_list_values.

        Parameters
        ----------
        metric_label: str
            Choice of metric. Currenlty only `snpcc` is accepted.
        """

        if metric_label == 'snpcc':
            self.metrics_list_names, self.metrics_list_values = \
                get_snpcc_metric(list(self.predicted_class),
                                 list(self.test_labels))
        else:
            raise ValueError('Only snpcc metric is implemented!'
                             '\n Feel free to add other options.')

    def make_query(self, strategy='UncSampling', batch=1,
                   dump=False) -> list:
        """Identify new object to be added to the training sample.

        Parameters
        ----------
        strategy: str (optional)
            Strategy used to choose the most informative object.
            Current implementation accepts 'UncSampling' and
            'RandomSampling'. Default is `UncSampling`.

        batch: int (optional)
            Number of objects to be chosen in each batch query.
            Default is 1.
        dump: bool (optional)
            If true, display on screen information about the
            displacement in order and classificaion probability due to
            constraints on queryable sample.

        Returns
        -------
        query_indx: list
            List of indexes identifying the objects to be queried in decreasing
            order of importance.
            If strategy=='RandomSampling' the order is irrelevant.
        """

        if strategy == 'UncSampling':
            query_indx = uncertainty_sampling(class_prob=self.classprob,
                                              queryable_ids=self.queryable_ids,
                                              test_ids=self.test_metadata['id'].values,
                                              batch=batch, dump=dump)
            return query_indx

        elif strategy == 'RandomSampling':
            query_indx = random_sampling(queryable_ids=self.queryable_ids,
                                         test_ids=self.test_metadata['id'].values,
                                         batch=batch)

            for n in query_indx:
                if self.test_metadata['id'].values[n] not in self.queryable_ids:
                    raise ValueError('Chosen object is not available for query!')

            return query_indx

        else:
            raise ValueError('Invalid strategy. Only "UncSampling" and '
                             '"RandomSampling are implemented! \n '
                             'Feel free to add other options. ')

    def update_samples(self, query_indx: list, loop: int, epoch=0):
        """Add the queried obj(s) to training and remove them from test.

        Update properties: train_headers, train_features, train_labels,
        test_labels, test_headers and test_features.

        Parameters
        ----------
        query_indx: list
            List of indexes identifying objects to be moved.
        loop: int
            Store number of loop when this query was made.
        """

        all_queries = []
        for obj in query_indx:
            # add object to the query sample
            query_header = self.test_metadata.values[obj]
            query_features = self.test_features[obj]
            line = [epoch]
            for item in query_header:
                line.append(item)
            for item1 in query_features:
                line.append(item1)

            # add object to the training sample
            new_header = pd.DataFrame([query_header], columns=self.metadata_names)
            self.train_metadata = pd.concat([self.train_metadata, new_header], axis=0,
                                            ignore_index=True)
            self.train_features = np.append(self.train_features,
                                            np.array([self.test_features[obj]]),
                                            axis=0)
            self.train_labels = np.append(self.train_labels,
                                          np.array([self.test_labels[obj]]),
                                          axis=0)

            # remove queried object from test sample
            self.test_metadata = self.test_metadata.drop(self.test_metadata.index[obj])
            self.test_labels = np.delete(self.test_labels, obj, axis=0)
            self.test_features = np.delete(self.test_features, obj, axis=0)

            all_queries.append(line)

        # update queried samples
        self.queried_sample.append(all_queries)

    def save_metrics(self, loop: int, output_metrics_file: str, epoch: int, batch=1):
        """Save current metrics to file.

        If loop == 0 the 'output_metrics_file' will be created or overwritten.
        Otherwise results will be added to an existing 'output_metrics file'.

        Parameters
        ----------
        loop: int
            Number of learning loops finished at this stage.
        output_metrics_file: str
            Full path to file to store metrics results.
        batch: int
            Number of queries in each loop.
        epoch: int
            Days since the beginning of the survey.
        """

        # add header to metrics file
        if not os.path.exists(output_metrics_file) or loop == 0:
            with open(output_metrics_file, 'w') as metrics:
                metrics.write('loop ')
                for name in self.metrics_list_names:
                    metrics.write(name + ' ')
                for j in range(batch):
                    metrics.write('query_id' + str(j + 1))
                metrics.write('\n')

        # write to file
        with open(output_metrics_file, 'a') as metrics:
            metrics.write(str(epoch) + ' ')
            for value in self.metrics_list_values:
                metrics.write(str(value) + ' ')
            for j in range(batch):
                metrics.write(str(self.queried_sample[loop][j][1]) + ' ')
            metrics.write('\n')

    def save_queried_sample(self, queried_sample_file: str, loop: int,
                            full_sample=False, batch=1):
        """Save queried sample to file.

        Parameters
        ----------
        queried_sample_file: str
            Complete path to output file.
        loop: int
            Number of learning loops finished at this stage.
        full_sample: bool (optional)
            If true, write down a complete queried sample stored in
            property 'queried_sample'. Otherwise append 1 line per loop to
            'queried_sample_file'. Default is False.
        """

        if full_sample:
            full_header = self.metadata_names + self.features_names
            query_sample = pd.DataFrame(self.queried_sample, columns=full_header)
            query_sample.to_csv(queried_sample_file, sep=' ', index=False)

        elif isinstance(loop, int):
            if not os.path.exists(queried_sample_file) or loop == 0:
                # add header to query sample file
                full_header = self.metadata_names + self.features_names
                with open(queried_sample_file, 'w') as query:
                    query.write('day ')
                    for item in full_header:
                        query.write(item + ' ')
                    query.write('\n')

            # save query sample to file
            with open(queried_sample_file, 'a') as query:
                for batch in range(batch):
                    for elem in self.queried_sample[loop][batch]:
                        query.write(str(elem) + ' ')
                    query.write('\n')


def main():
    return None


if __name__ == '__main__':
    main()



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 11 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

from actsnclass.build_snpcc_canonical import build_snpcc_canonical
from actsnclass.build_snpcc_canonical import plot_snpcc_train_canonical

__all__ = ['main']

def main(user_choices):
    """Build canonical sample for SNPCC data set fitted with Bazin features.

    Parameters
    ----------
    -c : bool
        If True, compute metadata on SNR and peak mag.
    -d : str (optional)
        Path to raw data directory. Needed only if "compute == False".
    -f : str
        Path to features file.
    -i: str (optional)
        Path to read metadata on SNR and peak mag from file.
        It is only used if "compute == False".
    -m : str (optional)
        Path to output file where to store metadata on SNR and peak mag.
        Only used if "save == True".
    -o : str
        Path to store full canonical sample.
    -p : str (optional)
        File to store comparison plot.
        If not provided plot is shown on screen.
    -s :  bool
        If True, save to file metadata on SNR and peakmag.

    Examples
    -------
    Use directly from the command line:

    >>> build_canonical.py -c <if True compute metadata>
    >>>       -d <path to raw data dir>
    >>>       -f <input features file> -m <output file for metadata>
    >>>       -o <output file for canonical sample> -p <comparison plot file>
    >>>       -s <if True save metadata to file>

    """

    print(user_choices.compute)

    sample = build_snpcc_canonical(path_to_raw_data=user_choices.raw_data_dir,
                                   path_to_features=user_choices.features,
                                   compute=user_choices.compute,
                                   save=user_choices.save,
                                   output_info_file=user_choices.output_meta,
                                   input_info_file=user_choices.input_meta,
                                   output_canonical_file=user_choices.output)

    plot_snpcc_train_canonical(sample, user_choices.output_plot_file)


def str2bool(v):
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')


if __name__ == '__main__':

    # get input directory and output file name from user
    parser = argparse.ArgumentParser(description='actsnclass - '
                                                 'Build Canonical module')

    parser.add_argument('-c', '--compute', required=True, type=str2bool,
                        dest='compute', help='If True, compute metadata '
                                             'on SNR and peak mag.')
    parser.add_argument('-d', '--raw-data-dir', dest='raw_data_dir',
                        required=False, type=str, default=None,
                        help='Path to raw data directory. Needed only'
                             'if "compute == False".')
    parser.add_argument('-f', '--features', required=True, type=str,
                        dest='features', help='Path to features file.')
    parser.add_argument('-i', '--input', required=False,
                        type=str, default='', dest='input_meta',
                        help='Path to read metadata on SNR and peak mag'
                             'from file. It is only used if '
                             '"compute == False".')
    parser.add_argument('-m', '--output-meta-data-file', required=False,
                        dest='output_meta', type=str, default=None,
                        help='Path to output file where to store '
                             'metadata on SNR and peak mag. Only used'
                             'if "save == True".')
    parser.add_argument('-o', '--output', required=True, type=str,
                        dest='output', help='Path to store '
                                            'full canonical sample.')
    parser.add_argument('-p', '--output-plot-file', required=False, type=str,
                        default=None, help='File to store comparison plot.'
                                         'If not provided plot is '
                                         'shown on screen.',
                        dest='output_plot_file')
    parser.add_argument('-s', '--save', required=True, type=str2bool,
                        dest='save', help='If True, save to file metadata'
                                          'on SNR and peakmag.')

    # get input directory and output file name from user
    from_user = parser.parse_args()

    main(from_user)



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 12 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

from actsnclass.time_domain import SNPCCPhotometry

__all__ = ['main']


def main(user_choice):
    """Generates features files for a list of days of the survey.

    Parameters
    ----------
    -d: sequence
        Sequence of days since the begin of the survey to be processed.
    -p: str
        Complete path to raw data directory.
    -o: str
        Complete path to output time domain directory.

    Examples
    -------
    Use it directly from the command line.

    >>> build_time_domain.py -d 20 21 22 23 -p <path to raw data dir> -o <path to output time domain dir>
    """
    path_to_data = user_choice.raw_data_dir
    output_dir = user_choice.output
    day = user_choice.day_of_survey

    for item in day:
        data = SNPCCPhotometry()
        data.create_daily_file(output_dir=output_dir, day=item)
        data.build_one_epoch(raw_data_dir=path_to_data, day_of_survey=int(item),
                             time_domain_dir=output_dir)


if __name__ == '__main__':
    # get input directory and output file name from user
    parser = argparse.ArgumentParser(description='actsnclass - '
                                                 'Prepare Time Domain module')

    parser.add_argument('-d', '--day', dest='day_of_survey', required=True,
                        nargs='+', help='Day after survey starts.')
    parser.add_argument('-p', '--path-to-data', required=True, type=str,
                        dest='raw_data_dir',
                        help='Complete path to raw data directory.')
    parser.add_argument('-o', '--output', dest='output', required=True,
                        type=str, help='Path to output time domain directory.')

    # get input directory and output file name from user
    from_user = parser.parse_args()

    main(from_user)



# Copyright 2019 actsnclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 8 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .bazin import bazin, fit_scipy
from .build_snpcc_canonical import build_snpcc_canonical
from .build_snpcc_canonical import Canonical, plot_snpcc_train_canonical
from .classifiers import random_forest
from .database import DataBase
from .fit_lightcurves import fit_snpcc_bazin, LightCurve
from .learn_loop import learn_loop
from .metrics import accuracy, efficiency, fom, purity, get_snpcc_metric
from .query_strategies import random_sampling, uncertainty_sampling
from .plot_results import Canvas
from .scripts.build_canonical import main as build_canonical
from .scripts.build_time_domain import main as build_time_domain
from .scripts.fit_dataset import main as fit_dataset
from .scripts.make_diagnostic_plots import main as make_diagnostic_plots
from .scripts.run_loop import main as run_loop
from .scripts.run_time_domain import main as run_time_domain
from .time_domain import SNPCCPhotometry


__all__ = ['accuracy',
           'bazin',
           'build_canonical',
           'build_snpcc_canonical',
           'Canonical',
           'Canvas',
           'DataBase',
           'efficiency',
           'fit_dataset',
           'fit_scipy',
           'fit_snpcc_bazin',
           'fom',
           'get_snpcc_metric',
           'learn_loop',
           'LightCurve',
           'make_diagnostic_plots',
           'plot_snpcc_train_canonical',
           'purity',
           'random_forest',
           'random_sampling',
           'run_loop',
           'run_time_domain',
           'SNPCCPhotometry',
           'uncertainty_sampling']



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 10 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = ['time_domain_loop', 'get_original_training']

from actsnclass import DataBase


def get_original_training(path_to_features, method='Bazin', screen=False):
    """Read original full light curve training sample

    Parameters
    ----------
    path_to_features: str
        Complete path to file holding full light curve features.
    method: str (optional)
        Feature extraction method. Only option implemented is "Bazin".
    screen: bool (optional)
        If true, show on screen comments on the dimensions of some key elements.

    Returns
    -------
    snactclass.DataBase
        Information about the original full light curve analys.
    """

    data = DataBase()
    data.load_features(path_to_features, method=method, screen=screen)
    data.build_samples(initial_training='original')

    return data


def time_domain_loop(days: list,  output_diag_file: str,
                     output_queried_file: str,
                     path_to_features_dir: str, strategy: str,
                     batch=1, canonical = False,  classifier='RandomForest',
                     features_method='Bazin', path_to_canonical="",
                     path_to_full_lc_features="",
                     screen=True, training='original'):
    """Perform the active learning loop. All results are saved to file.

    Parameters
    ----------
    days: list
        List of 2 elements. First and last day of observations since the
        beginning of the survey.
    output_diag_file: str
        Full path to output file to store diagnostics of each loop.
    output_queried_file: str
        Full path to output file to store the queried sample.
    path_to_features_dir: str
        Complete path to directory holding features files for all days.
    strategy: str
        Query strategy. Options are 'UncSampling' and 'RandomSampling'.
    batch: int (optional)
        Size of batch to be queried in each loop. Default is 1.
    canonical: bool (optional)
        If True, restrict the search to the canonical sample.
    classifier: str (optional)
        Machine Learning algorithm.
        Currently only 'RandomForest' is implemented.
    features_method: str (optional)
        Feature extraction method. Currently only 'Bazin' is implemented.
    path_to_canonical: str (optional)
        Path to canonical sample features files.
        It is only used if "strategy==canonical".
    path_to_full_lc_features: str (optional)
        Path to full light curve features file.
        Only used if training is a number.
    screen: bool (optional)
        If True, print on screen number of light curves processed.
    training: str or int (optional)
        Choice of initial training sample.
        If 'original': begin from the train sample flagged in the file
        If int: choose the required number of samples at random,
        ensuring that at least half are SN Ia
        Default is 'original'.
    """

    # initiate object
    data = DataBase()

    # load features for the first day
    path_to_features = path_to_features_dir + 'day_' + str(int(days[0])) + '.dat'
    data.load_features(path_to_features, method=features_method,
                       screen=screen)

    # change training
    if training == 'original':
        data.build_samples(initial_training='original')
        full_lc_features = get_original_training(path_to_features=path_to_full_lc_features)
        data.train_metadata = full_lc_features.train_metadata
        data.train_labels = full_lc_features.train_labels
        data.train_features = full_lc_features.train_features

    else:
        data.build_samples(initial_training=int(training))

    # get list of canonical ids
    if canonical:
        canonical = DataBase()
        canonical.load_features(path_to_file=path_to_canonical)
        data.queryable_ids = canonical.queryable_ids


    for night in range(int(days[0]), int(days[-1]) - 1):

        if screen:
            print('Processing night: ', night)

        # cont loop
        loop = night - int(days[0])

        # classify
        data.classify(method=classifier)

        # calculate metrics
        data.evaluate_classification()

        # choose object to query
        indx = data.make_query(strategy=strategy, batch=batch)

        # update training and test samples
        data.update_samples(indx, loop=loop)

        # save diagnostics for current state
        data.save_metrics(loop=loop, output_metrics_file=output_diag_file,
                          batch=batch, epoch=night)

        # save query sample to file
        data.save_queried_sample(output_queried_file, loop=loop,
                                 full_sample=False)

        # load features for next day
        path_to_features2 = path_to_features_dir + 'day_' + str(night + 1) + '.dat'

        data_tomorrow = DataBase()
        data_tomorrow.load_features(path_to_features2, method=features_method,
                                    screen=False)

        # notice that original here corresponds to original in the file
        # not original full light curve training
        data_tomorrow.build_samples('original')

        # use new test data
        data.test_metadata = data_tomorrow.test_metadata
        data.test_labels = data_tomorrow.test_labels
        data.test_features = data_tomorrow.test_features

        if strategy == 'canonical':
            data.queryable_ids = canonical.queryable_ids
        else:
            data.queryable_ids = data_tomorrow.queryable_ids


def main():
    return None


if __name__ == '__main__':
    main()



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 10 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = ['learn_loop']

from actsnclass import DataBase


def learn_loop(nloops: int, strategy: str, path_to_features: str,
               output_diag_file: str, output_queried_file: str,
               features_method='Bazin', classifier='RandomForest',
               training='original', batch=1, screen=True):
    """Perform the active learning loop. All results are saved to file.

    Parameters
    ----------
    nloops: int
        Number of active learning loops to run.
    strategy: str
        Query strategy. Options are 'UncSampling' and 'RandomSampling'.
    path_to_features: str
        Complete path to input features file.
    output_diag_file: str
        Full path to output file to store diagnostics of each loop.
    output_queried_file: str
        Full path to output file to store the queried sample.
    features_method: str (optional)
        Feature extraction method. Currently only 'Bazin' is implemented.
    classifier: str (optional)
        Machine Learning algorithm.
        Currently only 'RandomForest' is implemented.
    training: str or int (optional)
        Choice of initial training sample.
        If 'original': begin from the train sample flagged in the file
        If int: choose the required number of samples at random,
        ensuring that at least half are SN Ia
        Default is 'original'.
    batch: int (optional)
        Size of batch to be queried in each loop. Default is 1.
    screen: bool (optional)
        If True, print on screen number of light curves processed.
    """

    # initiate object
    data = DataBase()

    # load features
    data.load_features(path_to_features, method=features_method,
                       screen=screen)

    # separate training and test samples
    data.build_samples(initial_training=training)

    for loop in range(nloops):

        if screen:
            print('Processing... ', loop)

        # classify
        data.classify(method=classifier)

        # calculate metrics
        data.evaluate_classification()

        # choose object to query
        indx = data.make_query(strategy=strategy, batch=batch)

        # update training and test samples
        data.update_samples(indx, loop=loop)

        # save diagnostics for current state
        data.save_metrics(loop=loop, output_metrics_file=output_diag_file,
                          batch=batch, epoch=loop)

        # save query sample to file
        data.save_queried_sample(output_queried_file, loop=loop,
                                 full_sample=False)


def main():
    return None


if __name__ == '__main__':
    main()



# Copyright 2019 actsnclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 8 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .build_canonical import main as build_canonical
from .build_time_domain import main as build_time_domain
from .fit_dataset import main as fit_dataset
from .make_diagnostic_plots import main as make_diagnostic_plots
from .run_loop import main as run_loop
from .run_time_domain import main as run_time_domain


__all__ = ['build_canonical',
           'build_time_domain',
           'fit_dataset',
           'make_diagnostic_plots',
           'run_loop',
           'run_time_domain']



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 10 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = ['main']

import argparse

from actsnclass.time_domain_loop import time_domain_loop


def main(user_choice):
    """Command line interface to the Time Domain Active Learning scenario.

    Parameters
    ----------
    -d: sequence
         List of 2 elements. First and last day of observations since the
         beginning of the survey.
    -m: str
        Full path to output file to store metrics of each loop.
    -q: str
        Full path to output file to store the queried sample.
    -f: str
        Complete path to directory holding features files for all days.
    -s: str
        Query strategy. Options are 'UncSampling' and 'RandomSampling'.
    -b: int (optional)
        Size of batch to be queried in each loop. Default is 1.
    -c: str (optional)
        Machine Learning algorithm.
        Currently only 'RandomForest' is implemented.
    -fm: str (optional)
        Feature extraction method. Currently only 'Bazin' is implemented.
    -sc: bool (optional)
        If True, display comment with size of samples on screen.
    -t: str or int
        Choice of initial training sample.
        If 'original': begin from the train sample flagged in the file
        If int: choose the required number of samples at random,
        ensuring that at least half are SN Ia
        Default is 'original'.

    Returns
    -------
    metric file: file
        File with metrics calculated in each iteration.
    queried file: file
        All objects queried during the search, in sequence.

    Examples
    --------
    Use directly from the command line.

    >>> run_time_domain.py -d <first day of survey> <last day of survey>
    >>>        -m <output metrics file> -q <output queried file> -f <features directory>
    >>>        -s <learning strategy> -fm <path to full light curve features >

    Be aware to check the default options as well!
    """

    # set parameters
    days = user_choice.days
    output_diag_file = user_choice.metrics
    output_query_file = user_choice.queried
    path_to_features_dir = user_choice.features_dir
    strategy = user_choice.strategy

    batch = user_choice.batch
    classifier = user_choice.classifier
    feature_method = user_choice.feat_method
    path_to_full_lc_features = user_choice.full_features
    screen = user_choice.screen
    training = user_choice.training

    # run time domain loop
    time_domain_loop(days=days, output_diag_file=output_diag_file,
                     output_queried_file=output_query_file,
                     path_to_features_dir=path_to_features_dir,
                     strategy=strategy, batch=batch, classifier=classifier,
                     features_method=feature_method,
                     path_to_full_lc_features=path_to_full_lc_features,
                     screen=screen, training=training)


def str2bool(v):
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')


if __name__ == '__main__':

    # get input directory and output file name from user
    parser = argparse.ArgumentParser(description='actsnclass - '
                                                 'Time Domain loop module')
    parser.add_argument('-d', '--days', dest='days', required=True,
                        help='First and last day of survey.',
                        nargs='+')
    parser.add_argument('-m', '--metrics', dest='metrics', required=True,
                        help='File to store metrics results.')
    parser.add_argument('-q', '--queried', dest='queried',
                        help='File to store queried sample.',
                        required=True)
    parser.add_argument('-f', '--feat-dir', dest='features_dir',
                        required=True, help='Path to directory '
                                            'of feature files for each day.')
    parser.add_argument('-s', '--strategy', dest='strategy',
                        required=True,
                        help='Active Learning strategies. Possibilities are'
                             '"RandomSampling" or UncSampling.')
    parser.add_argument('-b', '--batch', dest='batch', required=False,
                        help='Number of queries in each iteration.',
                        default=1)
    parser.add_argument('-c', '--classifier', dest='classifier',
                        required=False, default='RandomForest',
                        help='Classifier. Currently only accepts '
                             '"RandomForest".')
    parser.add_argument('-fm', '--feature-method', dest='feat_method',
                        required=False, default='Bazin',
                        help='Feature extraction method. Currently only accepts '
                             '"Bazin".')
    parser.add_argument('-fl', '--full-light-curve',
                        dest='full_features', required=False,
                        default=' ', help='Path to full light curve features.'
                                          'Only used if '
                                          '"training==original".')
    parser.add_argument('-sc', '--screen', dest='screen', required=False,
                        default=True, type=str2bool,
                        help='If True, display size info on training and '
                             'test sample on screen.')
    parser.add_argument('-t', '--training', dest='training', required=False,
                        default='original', help='Choice of initial training'
                                                 'sample. It can be "original"'
                                                 'or an integer.')

    from_user = parser.parse_args()

    main(from_user)



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 10 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
import os
import pandas as pd
import matplotlib.pylab as plt

from sklearn.neighbors import NearestNeighbors, KernelDensity

from actsnclass.fit_lightcurves import LightCurve
from actsnclass.database import DataBase

__all__ = ['Canonical', 'build_snpcc_canonical', 'plot_snpcc_train_canonical']


class Canonical(object):
    """Canonical sample object.

    Attributes
    ----------
    canonical_ids: list
        List of ids for objects in the canonical sample.
    canonical_sample: list
        Complete data matrix for the canonical sample.
    meta_data: pd.DataFrame
        Metadata on sim peakmag and SNR for all objects in the original data set.
    test_ia_data: pd.DataFrame
        Metadata on sim peakmag and SNR for SN Ias in the test sample.
    test_ia_id: np.array
        Set of ids for all SN Ia in the test sample.
    test_ibc_data: pd.DataFrame
        Metadata on sim peakmag and SNR for SN Ibcs in the test sample.
    test_ibc_id: np.array
        Set of ids for all SN Ibc in the test sample.
    test_ii_data: pd.DataFrame
        Metadata on sim peakmag and SNR for SN IIs in the test sample.
    test_ii_id: np.array
        Set of ids for all SN II in the test sample.
    train_ia_data: pd.DataFrame
        Metadata on sim peakmag and SNR for SN Ias in the train sample.
    train_ia_id: np.array
        Set of ids for all SN Ia in the train sample.
    train_ibc_data: pd.DataFrame
        Metadata on sim peakmag and SNR for SN Ibcs in the train sample.
    train_ibc_id: np.array
        Set of ids for all SN Ibc in the train sample.
    train_ii_data: pd.DataFrame
        Metadata on sim peakmag and SNR for SN IIs in the train sample.
    train_ii_id: np.array
        Set of ids for all SN II in the train sample.

    Methods
    -------
    snpcc_get_canonical_info(path_to_rawdata_dir: str, canonical_output_file: st, compute: bool, save: bool, canonical_input_file: str)
        Load SNPCC metada data required to characterize objects.
    snpcc_identify_samples()
        Identify training and test sample.
    find_neighbors()
        Identify 1 nearest neighbor for each object in training.
    """

    def __init__(self):
        self.canonical_ids = []
        self.canonical_sample = []
        self.meta_data = pd.DataFrame()
        self.test_ia_data = pd.DataFrame()
        self.test_ia_id = np.array([])
        self.test_ibc_data = pd.DataFrame()
        self.test_ibc_id = np.array([])
        self.test_ii_data = pd.DataFrame()
        self.test_ii_id = np.array([])
        self.train_ia_data = pd.DataFrame()
        self.train_ia_id = np.array([])
        self.train_ibc_data = pd.DataFrame()
        self.train_ibc_id = np.array([])
        self.train_ii_data = pd.DataFrame()
        self.train_ii_id = np.array([])

    def snpcc_get_canonical_info(self, path_to_rawdata_dir: str,
                                 canonical_output_file: str,
                                 compute=True, save=True,
                                 canonical_input_file=''):
        """
        Load SNPCC metada data required to characterize objects.

        Populates attribute: data.

        Parameters
        ----------
        path_to_rawdata_dir: str
            Complete path to directory holding raw data files.
        canonical_output_file: str
            Complete path to output canonical sample file.
        compute: bool (optional)
            Compute required metada from raw data files.
            Default is True.
        save: bool (optional)
            Save metadata to file. Default is True.
        canonical_input_file: str (optional)
            Path to input file if required metadata was previously calculated.
            If name is give, 'compute' must be False.
        """

        if compute:
            # read file names
            file_list_all = os.listdir(path_to_rawdata_dir)
            lc_list = [elem for elem in file_list_all if 'DES_SN' in elem]

            sim_info_matrix = []

            for fname in lc_list:

                print('Processed for canonical: ', str(lc_list.index(fname)))

                # fit individual light curves
                lc = LightCurve()
                lc.load_snpcc_lc(path_to_rawdata_dir + fname)

                # store information for id, redshift and sim peak magnitude
                line = [lc.id, lc.sample, lc.sntype, lc.redshift]
                for val in lc.sim_peakmag:
                    line.append(val)

                # calculate mean SNR for each filter
                for f in lc.filters:
                    filter_flag = lc.photometry['band'].values == f
                    if sum(filter_flag) > 0:
                        snr = lc.photometry['SNR'].values[filter_flag]
                        mean_snr = np.mean(snr)
                        line.append(mean_snr)
                    else:
                        line.append(0)

                sim_info_matrix.append(line)

            metadata_header = ['snid', 'sample', 'sntype', 'z', 'g_pkmag',
                               'r_pkmag', 'i_pkmag', 'z_pkmag', 'g_SNR',
                               'r_SNR', 'i_SNR', 'z_SNR']
            self.meta_data = pd.DataFrame(sim_info_matrix,
                                          columns=metadata_header)

        else:
            if not canonical_input_file:
                raise ValueError('File not found! Set "calculate = True" '
                                 'to build canonical info file.')

            else:
                self.meta_data = pd.read_csv(canonical_input_file, sep=' ',
                                             index_col=False)

        if save:
            self.meta_data.to_csv(canonical_output_file, sep=' ', index=False)

    def snpcc_identify_samples(self):
        """Identify training and test sample.

        Populates attributes: train_ia_data, train_ia_id, train_ibc_data,
        train_ibc_id, train_ii_data, train_ibc_id, test_ia_data, test_ia_id,
        test_ibc_data, test_ibc_id, test_ii_data and test_ii_id.
        """

        # define parameters
        features_list = ['z', 'g_pkmag', 'r_pkmag', 'i_pkmag',
                         'z_pkmag', 'g_SNR', 'r_SNR', 'i_SNR', 'z_SNR']
        train_flag = self.meta_data['sample'].values == 'train'
        ia_flag = self.meta_data['sntype'].values == 'Ia'
        ibc_flag = self.meta_data['sntype'].values == 'Ibc'
        ii_flag = self.meta_data['sntype'].values == 'II'

        # get training sample
        self.train_ia_data = \
            self.meta_data[features_list][np.logical_and(train_flag, ia_flag)]
        self.train_ia_id = \
            self.meta_data['snid'].values[np.logical_and(train_flag, ia_flag)]
        self.train_ibc_data = \
            self.meta_data[features_list][np.logical_and(train_flag, ibc_flag)]
        self.train_ibc_id = \
            self.meta_data['snid'].values[np.logical_and(train_flag, ibc_flag)]
        self.train_ii_data = \
            self.meta_data[features_list][np.logical_and(train_flag, ii_flag)]
        self.train_ii_id = \
            self.meta_data['snid'].values[np.logical_and(train_flag, ii_flag)]

        # get test sample
        self.test_ia_data = \
            self.meta_data[features_list][np.logical_and(~train_flag, ia_flag)]
        self.test_ia_id = \
            self.meta_data['snid'].values[np.logical_and(~train_flag, ia_flag)]
        self.test_ibc_data = \
            self.meta_data[features_list][np.logical_and(~train_flag, ibc_flag)]
        self.test_ibc_id = \
            self.meta_data['snid'].values[np.logical_and(~train_flag, ibc_flag)]
        self.test_ii_data = \
            self.meta_data[features_list][np.logical_and(~train_flag, ii_flag)]
        self.test_ii_id = \
            self.meta_data['snid'].values[np.logical_and(~train_flag, ii_flag)]

    def find_neighbors(self):
        """Identify 1 nearest neighbor for each object in training.

        Populates attribute: canonical_ids.
        """

        # gather samples by type
        train_samples = [self.train_ia_data, self.train_ibc_data,
                         self.train_ii_data]
        test_samples = [self.test_ia_data, self.test_ibc_data,
                        self.test_ii_data]
        test_ids = [self.test_ia_id, self.test_ibc_id, self.test_ii_id]
        types = ['Ia', 'Ibc', 'II']

        # store the objects already chosen
        vault = []

        # find nearest neighbor
        for i in range(len(test_samples)):
            print('Scanning ', types[i], ' . . . ')

            # find 10 neighbors in case there are repetitions
            nbrs = NearestNeighbors(n_neighbors=10, algorithm='auto')
            nbrs.fit(test_samples[i])
            for j in range(train_samples[i].shape[0]):

                # search samples by type
                stype = np.array(train_samples[i].values[j]).reshape(1, -1)
                indices = nbrs.kneighbors(stype)

                # only add elements which were not added in a previous loop
                done = False
                count = 0
                while not done:
                    if indices[1][0][count] not in vault:
                        element = test_ids[i][indices[1][0][count]]
                        self.canonical_ids.append(element)
                        vault.append(indices[1][0][count])
                        done = True
                    else:
                        count = count + 1

            print('Processed: ', len(vault))


def build_snpcc_canonical(path_to_raw_data: str, path_to_features: str,
                          output_canonical_file: str, output_info_file='',
                          compute=True, save=True, input_info_file='',
                          features_method='Bazin'):
    """Build canonical sample for SNPCC data.

    Parameters
    ----------
    path_to_raw_data: str
        Complete path to raw data directory.
    path_to_features: str
        Complete path to Bazin features files.
    output_canonical_file: str
        Complete path to output canonical sample file.
    output_info_file: str
        Complete path to output metadata file for canonical sample.
        This includes SNR and simulated peak magnitude for each filter.
    compute: bool (optional)
        If True, compute metadata information on SNR and sim peak mag.
        If False, read info from file.
        Default is True.
    save: bool (optional)
        Save simulation metadata information to file.
        Default is True.
    input_info_file: str (optional)
        Complete path to sim metadata file.
        This must be provided if save == False.
    features_method: str (optional)
        Method for feature extraction. Only 'Bazin' is implemented.

    Returns
    -------
    actsnclass.Canonical: obj
        Updated canonical object with the attribute 'canonical_sample'.
    """

    # initiate canonical object
    sample = Canonical()

    # get necessary info
    sample.snpcc_get_canonical_info(path_to_rawdata_dir=path_to_raw_data,
                                    canonical_output_file=output_info_file,
                                    canonical_input_file=input_info_file,
                                    compute=compute, save=save)

    sample.snpcc_identify_samples()                 # identify samples
    sample.find_neighbors()                         # find neighbors

    # get metadata from features file
    data = DataBase()
    data.load_features(path_to_features, method=features_method)
    sample.header = data.metadata

    # identify new samples
    for i in range(data.data.shape[0]):
        if data.data.iloc[i]['id'] in sample.canonical_ids:
            data.data.at[i, 'sample'] = 'queryable'

    # save to file
    data.data.to_csv(output_canonical_file, sep=' ', index=False)

    # update Canonical object
    sample.canonical_sample = data.data

    print('Built canonical sample!')

    return sample


def plot_snpcc_train_canonical(sample: Canonical, output_plot_file=False):
    """Plot comparison between training and canonical samples.

    Parameters
    ----------
    sample: actsnclass.Canonical
        Canonical object holding infor for canonical sample
    output_plot_file: str (optional)
        Complete path to output plot.
        If not provided, plot is displayed on screen only.
    """

    # prepare data
    ztrain = sample.meta_data['z'][sample.meta_data['sample'].values == 'train']
    ztrain_axis = np.linspace(min(ztrain) - 0.25, max(ztrain) + 0.25, 1000)[:, np.newaxis]
    kde_ztrain = KernelDensity(kernel='gaussian', bandwidth=0.05).fit(np.array(ztrain).reshape(-1, 1))
    log_dens_ztrain = kde_ztrain.score_samples(ztrain_axis)

    canonical_flag = sample.canonical_sample['sample'] == 'queryable'
    zcanonical = sample.canonical_sample[canonical_flag]['redshift'].values
    zcanonical_axis = np.linspace(min(zcanonical) - 0.25, max(zcanonical) + 0.25, 1000)[:, np.newaxis]
    kde_zcanonical = KernelDensity(kernel='gaussian', bandwidth=0.05).fit(np.array(zcanonical).reshape(-1, 1))
    log_dens_zcanonical = kde_zcanonical.score_samples(zcanonical_axis)

    gtrain = sample.meta_data['g_pkmag'][sample.meta_data['sample'] == 'train'].values
    gtrain_axis = np.linspace(min(gtrain) - 0.25, max(gtrain) + 0.25, 1000)[:, np.newaxis]
    kde_gtrain = KernelDensity(kernel='gaussian', bandwidth=0.5).fit(np.array(gtrain).reshape(-1, 1))
    log_dens_gtrain = kde_gtrain.score_samples(gtrain_axis)

    gcanonical = [sample.meta_data['g_pkmag'].values[i] for i in range(sample.meta_data.shape[0])
                  if sample.meta_data['snid'][i] in sample.canonical_ids]
    gcanonical_axis = np.linspace(min(gcanonical) - 0.25, max(gcanonical) + 0.25, 1000)[:, np.newaxis]
    kde_gcanonical = KernelDensity(kernel='gaussian', bandwidth=0.5).fit(np.array(gcanonical).reshape(-1, 1))
    log_dens_gcanonical = kde_gcanonical.score_samples(gcanonical_axis)

    rtrain = sample.meta_data['r_pkmag'][sample.meta_data['sample'] == 'train'].values
    rtrain_axis = np.linspace(min(rtrain) - 0.25, max(rtrain) + 0.25, 1000)[:, np.newaxis]
    kde_rtrain = KernelDensity(kernel='gaussian', bandwidth=0.5).fit(np.array(rtrain).reshape(-1, 1))
    log_dens_rtrain = kde_rtrain.score_samples(rtrain_axis)

    rcanonical = [sample.meta_data['r_pkmag'].values[i] for i in range(sample.meta_data.shape[0])
                  if sample.meta_data['snid'][i] in sample.canonical_ids]
    rcanonical_axis = np.linspace(min(rcanonical) - 0.25, max(rcanonical) + 0.25, 1000)[:, np.newaxis]
    kde_rcanonical = KernelDensity(kernel='gaussian', bandwidth=0.5).fit(np.array(rcanonical).reshape(-1, 1))
    log_dens_rcanonical = kde_rcanonical.score_samples(rcanonical_axis)

    plt.figure(figsize=(20, 7))
    plt.subplot(1, 3, 1)
    plt.plot(ztrain_axis, np.exp(log_dens_ztrain), label='original train')
    plt.plot(zcanonical_axis, np.exp(log_dens_zcanonical), label='canonical')
    plt.xlabel('redshift', fontsize=20)
    plt.ylabel('PDF', fontsize=20)
    plt.legend(loc='upper right', fontsize=15)

    plt.subplot(1, 3, 2)
    plt.plot(gtrain_axis, np.exp(log_dens_gtrain), label='original train')
    plt.plot(gcanonical_axis, np.exp(log_dens_gcanonical), label='canonical')
    plt.xlabel('g_peakmag', fontsize=20)
    plt.ylabel('PDF', fontsize=20)
    plt.xlim(15, 35)
    plt.legend(loc='upper right', fontsize=15)

    plt.subplot(1, 3, 3)
    plt.plot(rtrain_axis, np.exp(log_dens_rtrain), label='original train')
    plt.plot(rcanonical_axis, np.exp(log_dens_rcanonical), label='canonical')
    plt.xlabel('r_peakmag', fontsize=20)
    plt.ylabel('PDF', fontsize=20)
    plt.xlim(15, 35)
    plt.legend(loc='upper right', fontsize=15)

    plt.tight_layout()
    plt.subplots_adjust(left=0.05, right=0.99, wspace=0.2)

    if output_plot_file:
        plt.savefig(output_plot_file)
        plt.close('all')
    else:
        plt.show()


def main():
    return None


if __name__ == '__main__':
    main()



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 10 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = ['learn_loop', 'main']

from actsnclass.learn_loop import learn_loop

import argparse


def main(args):
    """Command line interface to run the active learning loop.

    Parameters
    ----------
    -d: str
        Full path to output file to store diagnostics of each loop.
    -i: str
        Complete path to input features file.
    -n: int
        Number of active learning loops to run.
    -q: str
        Full path to output file to store the queried sample.
    -s: str
        Query strategy. Options are 'UncSampling' and 'RandomSampling'.
    -b: int (optional)
       Size of batch to be queried in each loop. Default is 1.
    -c: str (optional)
        Machine Learning algorithm.
        Currently only 'RandomForest' is implemented.
    -m: str (optional)
        Feature extraction method. Currently only 'Bazin' is implemented.
    -t: str or int (optional)
       Choice of initial training sample.
       If 'original': begin from the train sample flagged in the file
       If int: choose the required number of samples at random,
       ensuring that at least half are SN Ia.
       Default is 'original'.

    Examples
    --------

    Run directly from the command line:

    >>> run_loop.py -i <input features file> -b <batch size> -n <number of loops>
    >>>             -d <output metrics file> -q <output queried sample file>
    >>>             -s <learning strategy> -t <choice of initial training>

    """

    # set training sample variable
    if args.training == 'original':
        train = 'original'
    elif isinstance(int(args.training), int):
        train = int(args.training)
    else:
        raise ValueError('-t or --training option must be '
                         '"original" or integer!')

    # run active learning loop
    learn_loop(nloops=args.nquery, features_method=args.method,
               classifier=args.classifier,
               strategy=args.strategy, path_to_features=args.input,
               output_diag_file=args.diagnostics,
               output_queried_file=args.queried,
               training=train, batch=args.batch)


if __name__ == '__main__':

    # get input directory and output file name from user
    parser = argparse.ArgumentParser(description='actsnclass - '
                                                 'Learn loop module')
    parser.add_argument('-b', '--batch', dest='batch', required=True,
                        help='Number of samples to query in each loop.',
                        type=int)
    parser.add_argument('-c', '--classifier', dest='classifier',
                        help='Choice of machine learning classification.'
                             'Only "RandomForest" is implemented.'
                             'algorithm.', required=False, default='RandomForest',
                        type=str)
    parser.add_argument('-d', '--diagnostics', dest='diagnostics',
                        help='Path to output metrics file.', required=True,
                        type=str)
    parser.add_argument('-i', '--input', dest='input',
                        help='Path to features file.', required=True, type=str)
    parser.add_argument('-m', '--method', dest='method',
                        help='Feature extraction method. '
                             'Only "Bazin" is implemented.', required=False,
                        default='Bazin', type=str)
    parser.add_argument('-n', '--nquery', dest='nquery', required=True,
                        help='Number of query loops to run.', type=int)
    parser.add_argument('-q', '--queried', dest='queried', type=str,
                        help='Path to output queried sample file.', required=True)
    parser.add_argument('-s', '--strategy', dest='strategy',
                        help='Active Learning strategy. Options are '
                             '"UncSampling" and "RanomSampling".',
                        required=True, type=str)
    parser.add_argument('-t', '--training', dest='training', required=True,
                        help='Initial training sample. Options are '
                             '"original" or integer.', type=str)

    from_user = parser.parse_args()

    main(from_user)



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 9 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = ['efficiency', 'purity', 'fom', 'accuracy', 'get_snpcc_metric']


def efficiency(label_pred: list, label_true: list, ia_flag=1):
    """Calculate efficiency.

    Parameters
    ----------
    label_pred: list
        Predicted labels
    label_true: list
        True labels
    ia_flag: int (optional)
        Flag used to identify Ia objects. Default is 1.

    Returns
    -------
    efficiency: float
       Fraction of correctly classified SN Ia.

    """

    cc_ia = sum([label_pred[i] == label_true[i] and label_true[i] == ia_flag for i in range(len(label_pred))])
    tot_ia = sum([label_true[i] == ia_flag for i in range(len(label_true))])

    return float(cc_ia) / tot_ia


def purity(label_pred: list, label_true: list, ia_flag=1):
    """ Calculate purity.

    Parameters
    ----------
    label_pred: list
        Predicted labels
    label_true: list
        True labels
    ia_flag: int (optional)
        Flag used to identify Ia objects. Default is 1.

    Returns
    -------
    Purity: float
        Fraction of true SN Ia in the final classified Ia sample.

    """

    cc_ia = sum([label_pred[i] == label_true[i] and label_true[i] == ia_flag for i in range(len(label_pred))])
    wr_nia = sum([label_pred[i] != label_true[i] and label_true[i] != ia_flag for i in range(len(label_pred))])

    if cc_ia + wr_nia > 0:
        return float(cc_ia) / (cc_ia + wr_nia)
    else:
        return 0


def fom(label_pred: list, label_true: list, ia_flag=1, penalty=3.0):
    """
    Calculate figure of merit.

    Parameters
    ----------
    label_pred: list
        Predicted labels
    label_true: list
        True labels
    ia_flag: bool (optional)
        Flag used to identify Ia objects. Default is 1.
    penalty: float
        Weight given for non-Ias wrongly classified.

    Returns
    -------
    figure of merit: float
        Efficiency x pseudo-purity (purity with a penalty for false positives)

    """

    cc_ia = sum([label_pred[i] == label_true[i] and label_true[i] == ia_flag for i in range(len(label_pred))])
    wr_nia = sum([label_pred[i] != label_true[i] and label_true[i] != ia_flag for i in range(len(label_pred))])
    tot_ia = sum([label_true[i] == ia_flag for i in range(len(label_true))])

    if (cc_ia + penalty * wr_nia) > 0:
        return (float(cc_ia) / (cc_ia + penalty * wr_nia)) * float(cc_ia) / tot_ia
    else:
        return 0


def accuracy(label_pred: list, label_true: list):
    """Calculate accuracy.

    Parameters
    ----------
    label_pred: list
        predicted labels
    label_true: list
        true labels

    Returns
    -------
    Accuracy: float
        Global fraction of correct classifications.

    """

    cc = sum([label_pred[i] == label_true[i] for i in range(len(label_pred))])

    return cc / len(label_pred)


def get_snpcc_metric(label_pred: list, label_true: list, ia_flag=1,
                     wpenalty=3):
    """
    Calculate the metric parameters used in the SNPCC.

    Parameters
    ----------
    label_pred: list
        Predicted labels
    label_true: list
        True labels
    ia_flag: bool (optional)
        Flag used to identify Ia objects. Default is 1.
    wpenalty: float
        Weight given for non-Ias wrongly classified.


    Returns
    -------
    metric_names: list
        Name of elements in metrics: [accuracy, eff, purity, fom]
    metric_values: list
        list of calculated metrics values for each element

    """

    calc_eff = efficiency(label_pred=label_pred,
                          label_true=label_true, ia_flag=ia_flag)
    calc_purity = purity(label_pred=label_pred,
                         label_true=label_true, ia_flag=ia_flag)
    calc_fom = fom(label_pred=label_pred,
                   label_true=label_true, ia_flag=ia_flag, penalty=wpenalty)
    calc_accuracy = accuracy(label_pred=label_pred, label_true=label_true)

    metric_values = [calc_accuracy, calc_eff, calc_purity, calc_fom]
    metric_names = ['accuracy', 'efficiency', 'purity', 'fom']

    return metric_names, metric_values


def main():
    return None


if __name__ == '__main__':
    main()



# Copyright 2019 actsnclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 8 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

from actsnclass.fit_lightcurves import fit_snpcc_bazin

__all__ = ['main']


def main(user_choices):
    """Fit the entire sample with the Bazin function.

    All results are saved to file.

    Parameters
    ----------
    -dd: str
        Path to directory containing raw data.
    -o: str
        Path to output feature file.

    Examples
    --------

    Run directly from the command line.

    >>> fit_dataset.py -dd <path_to_data_dir> -o <output_file>
    """

    # raw data directory
    data_dir = user_choices.input
    features_file = user_choices.output

    # fit the entire sample
    fit_snpcc_bazin(path_to_data_dir=data_dir, features_file=features_file)

    return None


if __name__ == '__main__':

    # get input directory and output file name from user
    parser = argparse.ArgumentParser(description='actsnclass - Fit Light curves module')
    parser.add_argument('-dd', '--datadir', dest='input',
                        help='Path to directory holding raw data.', required=True)
    parser.add_argument('-o', '--output', dest='output', help='Path to output file.', required=True)

    user_input = parser.parse_args()

    main(user_input)



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 9 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from actsnclass.bazin import bazin, fit_scipy

import matplotlib.pylab as plt
import numpy as np
import os
import pandas as pd

__all__ = ['LightCurve', 'fit_snpcc_bazin']


class LightCurve(object):
    """ Light Curve object, holding meta and photometric data.

    Attributes
    ----------
    bazin_features_names: list
        List of names of the Bazin function parameters.
    bazin_features: list
        List with the 5 best-fit Bazin parameters in all filters.
        Concatenated from blue to red.
    dataset_name: str
        Name of the survey or data set being analyzed.
    filters: list
        List of broad band filters.
    id: int
        SN identification number
    photometry: pd.DataFrame
        Photometry information. Keys --> [mjd, band, flux, fluxerr, SNR, MAG, MAGERR].
    redshift: float
        Redshift
    sample: str
        Original sample to which this light curve is assigned
    sim_peakmag: np.array
        Simulated peak magnitude in each filter
    sncode: int
        Number identifying the SN model used in the simulation
    sntype: str
        General classification, possibilities are: Ia, II or Ibc

    Methods
    -------
    check_queryable(mjd: float, r_lim: float)
        Check if this light can be queried in a given day.
    load_snpcc_lc(path_to_data: str)
        Reads header and photometric information for 1 light curve
    fit_bazin(band: str) -> list
        Calculates best-fit parameters from the Bazin function in 1 filter
    fit_bazin_all()
        Calculates  best-fit parameters from the Bazin func for all filters
    plot_bazin_fit(save: bool, show: bool, output_file: srt)
        Plot photometric points and Bazin fitted curve

    Examples
    --------
    >>> from actsnclass import LightCurve

    define path to light curve file

    >>> path_to_lc = 'data/SIMGEN_PUBLIC_DES/DES_SN431546.DAT'

    >>> lc = LightCurve()                        # create light curve instance
    >>> lc.load_snpcc_lc(path_to_lc)             # read data
    >>> lc.photometry                            # display photometry
              mjd band     flux  fluxerr   SNR
    0   56207.188    g   9.6560    4.369  2.21
    1   56207.195    r   6.3370    3.461  1.83
    ...        ...  ...      ...      ...   ...
    96  56336.043    r  14.4300    3.098  4.66
    97  56336.055    i  18.9500    5.029  3.77
    [98 rows x 5 columns]

    >>> lc.fit_bazin_all()                  # perform Bazin fit in all filters
    >>> lc.bazin_features                   # display Bazin parameters
    [62.0677260096896, -7.959383808822104, 47.37511467606875, 37.4919069623379,
    ... ... ...
    206.65806244385922, -4.777010246622081]

    plot light curve fit

    >>> lc.plot_bazin_fit(output_file=str(lc.id) + '.png')

    for fitting the entire sample...

    >>> path_to_data_dir = 'data/SIMGEN_PUBLIC_DES/'     # raw data directory
    >>> output_file = 'results/Bazin.dat'       # output file
    >>> fit_snpcc_bazin(path_to_data_dir=path_to_data_dir, features_file=output_file)

    a file with all Bazin fits for this data set was produced
    """

    def __init__(self):
        self.bazin_features = []
        self.bazin_features_names = ['a', 'b', 't0', 'tfall', 'trsise']
        self.dataset_name = ' '
        self.filters = []
        self.id = 0
        self.photometry = pd.DataFrame()
        self.redshift = 0
        self.sample = ' '
        self.sim_peakmag = []
        self.sncode = 0
        self.sntype = ' '

    def load_snpcc_lc(self, path_to_data: str):
        """Reads one LC from SNPCC data.

        Populates the attributes: dataset_name, id, sample, redshift, sncode,
        sntype, photometry and sim_peakmag.

        Parameters
        ---------
        path_to_data: str
            Path to text file with data from a single SN.
        """

        # set the designation of the data set
        self.dataset_name = 'SNPCC'

        # set filters
        self.filters = ['g', 'r', 'i', 'z']

        # set SN types
        snii = ['2', '3', '4', '12', '15', '17', '19', '20', '21', '24', '25',
                '26', '27', '30', '31', '32', '33', '34', '35', '36', '37',
                '38', '39', '40', '41', '42', '43', '44']

        snibc = ['1', '5', '6', '7', '8', '9', '10', '11', '13', '14', '16',
                 '18', '22', '23', '29', '45', '28']

        # read light curve data
        op = open(path_to_data, 'r')
        lin = op.readlines()
        op.close()

        # separate elements
        data_all = np.array([elem.split() for elem in lin])

        # flag useful lines
        flag_lines = np.array([True if len(line) > 1 else False for line in data_all])

        # get only informative lines
        data = data_all[flag_lines]

        photometry_raw = []               # store photometry
        header = []                      # store parameter header

        # get header information
        for line in data:
            if line[0] == 'SNID:':
                self.id = int(line[1])
            elif line[0] == 'SNTYPE:':
                if line[1] == '-9':
                    self.sample = 'test'
                else:
                    self.sample = 'train'
            elif line[0] == 'SIM_REDSHIFT:':
                self.redshift = float(line[1])
            elif line[0] == 'SIM_NON1a:':
                self.sncode = line[1]
                if line[1] in snibc:
                    self.sntype = 'Ibc'
                elif line[1] in snii:
                    self.sntype = 'II'
                elif line[1] == '0':
                    self.sntype = 'Ia'
                else:
                    raise ValueError('Unknown supernova type!')
            elif line[0] == 'VARLIST:':
                header: list = line[1:]
            elif line[0] == 'OBS:':
                photometry_raw.append(np.array(line[1:]))
            elif line[0] == 'SIM_PEAKMAG:':
                self.sim_peakmag = np.array([float(item) for item in line[1:5]])

        # transform photometry into array
        photometry_raw = np.array(photometry_raw)

        # put photometry into data frame
        self.photometry['mjd'] = np.array([float(item) for item in photometry_raw[:, header.index('MJD')]])
        self.photometry['band'] = np.array(photometry_raw[:, header.index('FLT')])
        self.photometry['flux'] = np.array([float(item) for item in photometry_raw[:, header.index('FLUXCAL')]])
        self.photometry['fluxerr'] = np.array([float(item) for item in photometry_raw[:, header.index('FLUXCALERR')]])
        self.photometry['SNR'] = np.array([float(item) for item in photometry_raw[:, header.index('SNR')]])
        self.photometry['MAG'] = np.array([float(item) for item in photometry_raw[:, header.index('MAG')]])
        self.photometry['MAGERR'] = np.array([float(item) for item in photometry_raw[:, header.index('MAGERR')]])

    def check_queryable(self, mjd: float, r_lim: float):
        """Check if this light can be queried in a given day.

        This checks only r-band mag limit in a given epoch.
        It there is no observation on that day, use the last available
        observation.

        Parameters
        ----------
        mjd: float
            MJD where the query will take place.
        r_lim: float
            r-band magnitude limit below which query is possible.

        Returns
        -------
        bool
            If true, sample is changed to `queryable`.
        """

        # create photo flag
        photo_flag = self.photometry['mjd'].values <= mjd
        rband_flag = self.photometry['band'].values == 'r'
        surv_flag = np.logical_and(photo_flag, rband_flag)

        # check surviving photometry
        surv_mag = self.photometry['MAG'].values[surv_flag]

        if len(surv_mag) > 0 and 0 < surv_mag[-1] <= r_lim:
            return True
        else:
            return False

    def fit_bazin(self, band: str):
        """Extract Bazin features for one filter.

        Parameters
        ----------
        band: str
            Choice of broad band filter

        Returns
        -------
        bazin_param: list
            Best fit parameters for the Bazin function: [a, b, t0, tfall, trise]
        """

        # build filter flag
        filter_flag = self.photometry['band'] == band

        # get info for this filter
        time = self.photometry['mjd'].values[filter_flag]
        flux = self.photometry['flux'].values[filter_flag]

        # fit Bazin function
        bazin_param = fit_scipy(time - time[0], flux)

        return bazin_param

    def fit_bazin_all(self):
        """Perform Bazin fit for all filters independently and concatenate results.

        Populates the attributes: bazin_features.
        """

        for band in self.filters:
            # build filter flag
            filter_flag = self.photometry['band'] == band

            if sum(filter_flag) > 4:
                best_fit = self.fit_bazin(band)

                if sum([str(item) == 'nan' for item in best_fit]) == 0:
                    for fit in best_fit:
                        self.bazin_features.append(fit)
                else:
                    for i in range(5):
                        self.bazin_features.append('None')
            else:
                for i in range(5):
                    self.bazin_features.append('None')

    def plot_bazin_fit(self, save=True, show=False, output_file=' '):
        """
        Plot data and Bazin fitted function.

        Parameters
        ----------
        save: bool (optional)
             Save figure to file. Default is True.
        show: bool (optinal)
             Display plot in windown. Default is False.
        output_file: str
            Name of file to store the plot.
        """

        plt.figure()

        for i in range(len(self.filters)):
            plt.subplot(2, len(self.filters) / 2 + len(self.filters) % 2, i + 1)
            plt.title('Filter: ' + self.filters[i])

            # filter flag
            filter_flag = self.photometry['band'] == self.filters[i]
            x = self.photometry['mjd'][filter_flag].values
            y = self.photometry['flux'][filter_flag].values
            yerr = self.photometry['fluxerr'][filter_flag].values

            # shift to avoid large numbers in x-axis
            time = x - min(x)
            xaxis = np.linspace(0, max(time), 500)[:, np.newaxis]
            # calculate fitted function
            fitted_flux = np.array([bazin(t, self.bazin_features[i * 5],
                                          self.bazin_features[i * 5 + 1],
                                          self.bazin_features[i * 5 + 2],
                                          self.bazin_features[i * 5 + 3],
                                          self.bazin_features[i * 5 + 4])
                                    for t in xaxis])

            plt.errorbar(time, y, yerr=yerr, color='blue', fmt='o')
            plt.plot(xaxis, fitted_flux, color='red', lw=1.5)
            plt.xlabel('MJD - ' + str(min(x)))
            plt.ylabel('FLUXCAL')
            plt.tight_layout()

        if save:
            plt.savefig(output_file)
        if show:
            plt.show()


def fit_snpcc_bazin(path_to_data_dir: str, features_file: str):
    """Fit Bazin functions to all filters in training and test samples.

    Parameters
    ----------
    path_to_data_dir: str
        Path to directory containing the set of individual files, one for each light curve.
    features_file: str
        Path to output file where results should be stored.
    """

    # read file names
    file_list_all = os.listdir(path_to_data_dir)
    lc_list = [elem for elem in file_list_all if 'DES_SN' in elem]

    # count survivers
    count_surv = 0

    # add headers to files
    with open(features_file, 'w') as param_file:
        param_file.write('id redshift type code sample gA gB gt0 gtfall gtrise rA rB rt0 rtfall rtrise iA iB it0 ' +
                         'itfall itrise zA zB zt0 ztfall ztrise\n')

    for file in lc_list:

        # fit individual light curves
        lc = LightCurve()
        lc.load_snpcc_lc(path_to_data_dir + file)
        lc.fit_bazin_all()

        print(lc_list.index(file), ' - id:', lc.id)

        # append results to the correct matrix
        if 'None' not in lc.bazin_features:
            count_surv = count_surv + 1
            print('Survived: ', count_surv)

            # save features to file
            with open(features_file, 'a') as param_file:
                param_file.write(str(lc.id) + ' ' + str(lc.redshift) + ' ' + str(lc.sntype) + ' ')
                param_file.write(str(lc.sncode) + ' ' + str(lc.sample) + ' ')
                for item in lc.bazin_features:
                    param_file.write(str(item) + ' ')
                param_file.write('\n')

    param_file.close()


def main():
    return None


if __name__ == '__main__':
    main()



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 10 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

from actsnclass.plot_results import Canvas

__all__ = ['main']


def main(user_input):
    """Generate diagnostic plots.

    Parameters
    ----------
    -m: list
        List of paths to the diagnostic files. One path for each
        learning strategy we wish to plot. This must follow the
        same order as give in `-s`.
    -o: str
        Path to output file where the plot will be stored.
    -s: list
        List of keywords describing the learning strategies to be plotted.
        Order must be the same as provided in `-m`.
        Options are ['canonical', 'rand_sampling', 'unc_sampling']

    Examples
    --------
    Use it directly from the command line.
    For example, if you wish to make a diagnostic plot for the random sampling and
    uncertainty sampling together, do:

    >>> make_diagnostic_plots.py -m <path to rand sampling diag> <path to unc sampling diag>
    >>>     -o <path to output plot file> -s RandomSampling UncSampling

    """

    # create Canvas object
    cv = Canvas()

    # load data
    cv.load_diagnostics(path_to_files=list(user_input.metrics),
                        strategies_list=list(user_input.strategies))

    # set plot dimensions
    cv.set_plot_dimensions()

    # save plot to file
    cv.plot_diagnostics(output_plot_file=user_input.output,
                        strategies_list=list(user_input.strategies))


if __name__ == '__main__':

    # get input directory and output file name from user
    parser = argparse.ArgumentParser(description='actsnclass - '
                                                 'Learn loop module')
    parser.add_argument('-m', '--metrics-files-list', dest='metrics',
                        required=True, type=str,
                        help='List of path to diagnostics files for '
                             'different learning strategies.', nargs='+')
    parser.add_argument('-o', '--output-plot', dest='output',
                        required=True, type=str,
                        help='Complete path to output plot file.')
    parser.add_argument('-s', '--strategy-names-list', dest='strategies',
                        required=True, type=str, nargs='+',
                        help='List of strategies names. This should be'
                             'in the same order as the given list of '
                             'diagnostics files.')

    from_user = parser.parse_args()

    main(from_user)






# Copyright 2019 actsnclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 7 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import setuptools


setuptools.setup(
    name='actsnclass',
    version='1.2',
    packages=setuptools.find_packages(),
    py_modules=['bazin',
                'build_snpcc_canonical',
                'classifiers',
                'data_base',
                'fit_lightcurves',
                'learn_loop',
                'metrics',
                'plot_results',
                'query_strategies',
                'time_domain'],
    scripts=['actsnclass/scripts/build_canonical.py',
             'actsnclass/scripts/build_time_domain.py',
             'actsnclass/scripts/fit_dataset.py',
             'actsnclass/scripts/make_diagnostic_plots.py',
             'actsnclass/scripts/run_loop.py',
             'actsnclass/scripts/run_time_domain.py'],
    url='https://github.com/COINtoolbox/ActSNClass',
    license='GNU3',
    author='Emille E. O.Ishida',
    author_email='emille@cosmostatistics-initiative.org',
    description='Active Learning for SN photometric classification'
)



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 12 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from actsnclass import LightCurve


class SNPCCPhotometry(object):
    """Handles photometric information for entire SNPCC data.

    This class only works for Bazin feature extraction method.

    Attributes
    ----------
    bazin_header: str
        Reader to be added to features files for each day.
    max_epoch: float
        Maximum MJD for the entire data set.
    min_epoch: float
        Minimum MJD for the entire data set.
    rmag_lim: float
        Maximum r-band magnitude allowing a query.

    Methods
    -------
    get_lim_mjds(raw_data_dir)
        Get minimum and maximum MJD for complete sample.
    create_daily_file(raw_data_dir: str, day: int, output_dir: str, header: str)
        Create one file for a given day of the survey.
        Only populates the file with header. It will erase existing files!
    build_one_epoch(raw_data_dir: str, day_of_survey: int, time_domain_dir: str, feature_method: str, dataset: str)
        Selects objects with observed points until given MJD, performs feature extraction
        and evaluate if query is possible. Save results to file.
    """
    def __init__(self):
        self.bazin_header = 'id redshift type code sample gA gB gt0 ' + \
                            'gtfall gtrise rA rB rt0 rtfall rtrise iA ' + \
                            'iB it0 itfall itrise zA zB zt0 ztfall ztrise\n'
        self.max_epoch = 56352
        self.min_epoch = 56171
        self.rmag_lim = 24

    def get_lim_mjds(self, raw_data_dir):
        """Get minimum and maximum MJD for complete sample.

        This function is not necessary if you are working with
        SNPCC data. The values are hard coded in the class.

        Parameters
        ----------
        raw_data_dir: str
            Complete path to raw data directory.

        Returns
        -------
        limits: list
            List of extreme MJDs for entire sample: [min_MJD, max_MJD].
        """

        # read file names
        file_list_all = os.listdir(raw_data_dir)
        lc_list = [elem for elem in file_list_all if 'DES_SN' in elem]

        # store MJDs
        min_day = []
        max_day = []

        for elem in lc_list:

            print('Searching obj n. ', lc_list.index(elem))

            lc = LightCurve()                        # create light curve instance
            lc.load_snpcc_lc(raw_data_dir + elem)                      # read data

            min_day.append(min(lc.photometry['mjd'].values))
            max_day.append(max(lc.photometry['mjd'].values))

            self.min_epoch = min(min_day)
            self.max_epoch = max(max_day)

        return [min(min_day), max(max_day)]

    def create_daily_file(self, output_dir: str,
                          day: int, header='Bazin'):
        """Create one file for a given day of the survey.

        The file contains only header for the features file.

        Parameters
        ----------
        output_dir: str
            Complete path to raw data directory.
        day: int
            Day passed since the beginning of the survey.
        header: str (optional)
            List of elements to be added to the header.
            Separate by 1 space.
            Default option uses header for Bazin features file.
        """

        features_file = output_dir + 'day_' + str(day) + '.dat'

        if header == 'Bazin':
            # add headers to files
            with open(features_file, 'w') as param_file:
                param_file.write(self.bazin_header)

        else:
            with open(features_file, 'w') as param_file:
                param_file.write(self.header)

    def build_one_epoch(self, raw_data_dir: str, day_of_survey: int,
                        time_domain_dir: str, feature_method='Bazin',
                        dataset='SNPCC'):

        # read file names
        file_list_all = os.listdir(raw_data_dir)
        lc_list = [elem for elem in file_list_all if 'DES_SN' in elem]

        # get name of file to store results of today
        features_file = time_domain_dir + 'day_' + str(day_of_survey) + '.dat'

        # count survivers
        count_surv = 0

        for i in range(len(lc_list)):
            print('Processed : ', i)

            lc = LightCurve()  # create light curve instance

            if dataset == 'SNPCC':
                lc.load_snpcc_lc(raw_data_dir + lc_list[i])
            else:
                raise ValueError('Only SNPCC data set is implemented!')

            # see which epochs are observed for until this day
            today = day_of_survey + self.min_epoch
            photo_flag = lc.photometry['mjd'].values <= today

            # check if any point survived, other checks are made
            # inside lc object
            if sum(photo_flag) > 4:
                # only the allowed photometry
                lc.photometry = lc.photometry[photo_flag]

                # perform feature extraction
                if feature_method == 'Bazin':
                    lc.fit_bazin_all()
                else:
                    raise ValueError('Only Bazin features are implemented!')

                # only save to file if all filters were fitted
                if len(lc.bazin_features) > 0 and \
                        'None' not in lc.bazin_features:
                    count_surv = count_surv + 1
                    print('... ... ... Survived: ', count_surv)

                    # see if query is possible
                    queryable = \
                        lc.check_queryable(mjd=self.min_epoch + day_of_survey,
                                           r_lim=self.rmag_lim)

                    if queryable:
                        lc.sample = 'queryable'

                    # save features to file
                    with open(features_file, 'a') as param_file:
                        param_file.write(str(lc.id) + ' ' +
                                         str(lc.redshift) + ' ' +
                                         str(lc.sntype) + ' ')
                        param_file.write(str(lc.sncode) + ' ' +
                                         str(lc.sample) + ' ')
                        for item in lc.bazin_features[:-1]:
                            param_file.write(str(item) + ' ')
                        param_file.write(str(lc.bazin_features[-1]) + '\n')


# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 10 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import matplotlib.pylab as plt
import numpy as np
import pandas as pd
import seaborn as sns

__all__ = ['Canvas']


class Canvas(object):
    """Canvas object, handles and plot information from multiple strategies.

    Attributes
    ----------
    axis_label_size: int
        Size of font in axis labels.
    canonical: pd.DataFrame
        Data from Canonical strategy.
    fig_size: tuple
        Figure dimensions.
    rand_sampling: pd.DataFrame
        Data from Random Sampling strategy.
    tick_label_size: int
        Size of tick labels in both axis.
    ncolumns: int
        Number of columns in panel grid.
    nlines: int
        Number of lines in panel grid.
    nmetrics: int
        Number of metric elements (panels in plot).
    metrics_names: list
        List of names for metrics to be plotted.
    unc_sampling: pd.DataFrame
        Data from Uncertainty Sampling strategy.
    colors: dict
        Colors corresponding to each strategy.
        They were chosen to follow Mondrian's color palette.
        Do not change and try to keep the same palette in adding
        new elements.
    labels: dict
        Labels to appear on plot for each strategy.
    markers: dict
        Plot markers for each strategy.
    strategies: dict
        Dictionary connecting each data frame to its
        standard nomenclature (not the plot labels).

    Methods
    -------
    load_diagnostics(path_to_files: list, strategy_list: list)
        Load figure of merit diagnostics and identify set of metrics.
    set_plot_dimensions()
        Set directives for plot sizes based on number of metrics.
    plot_diagnostics(output_plot_file: str, strategies_list: list)
        Generate plot for all metrics in files and strategies given as input.

    Examples
    --------
    Define input variables

    >>> path_to_files = ['results/metrics_canonical.dat',
    >>>                  'results/metrics_random.dat',
    >>>                  'results/metrics_unc.dat']
    >>> strategies_list = ['Canonical', 'RandomSampling', 'UncSampling']
    >>> output_plot = 'plots/diag1_unc.png'

    Initiate the Canvas object, read and plot the results for
    each diagnostic and strategy.

    >>> cv = Canvas()
    >>> cv.load_diagnostics(path_to_files=path_to_files,
    >>>                    strategies_list=strategies_list)
    >>> cv.set_plot_dimensions()
    >>> cv.plot_diagnostics(output_plot_file=output_plot,
    >>>                    strategies_list=strategies_list)

    """

    def __init__(self):
        self.axis_label_size = 30
        self.canonical = pd.DataFrame()
        self.fig_size = ()
        self.rand_sampling = pd.DataFrame()
        self.tick_label_size = 22
        self.ncolumns = 0
        self.nlines = 2
        self.line_width = 5
        self.nmetrics = 0
        self.metrics_names = []
        self.unc_sampling = pd.DataFrame()
        self.colors = {'Canonical': '#dd0100',      # red
                       'RandomSampling': '#fac901',  # yellow
                       'UncSampling': '#225095'}   # blue
        self.labels = {'Canonical': 'Canonical',
                       'RandomSampling': 'Passive Learning',
                       'UncSampling': 'AL - Uncertainty Sampling'}
        self.markers = {'Canonical': '--',
                        'RandomSampling': ':',
                        'UncSampling': '-.'}
        self.strategies = {'Canonical': self.canonical,
                           'RandomSampling': self.rand_sampling,
                           'UncSampling': self.unc_sampling}

    def load_diagnostics(self, path_to_files: list, strategies_list: list):
        """Load figure of merit diagnostics and identify set of metrics.

        Populates attributes: canonical, unc_sampling or rand_sampling,
        depending on choice of 'sample'.

        Parameters
        ----------
        path_to_files : str
            List of paths to diagnostic files for different strategies.
        strategies_list: list
            List of all strategies to be included in the same plot.
            Current possibibilities are:
            ['canonical', 'rand_sampling', 'unc_sampling'].
        """

        # read data
        for i in range(len(strategies_list)):
            name = strategies_list[i]
            self.strategies[name] = pd.read_csv(path_to_files[i],
                                                sep=' ',
                                                index_col=False)

            # get metrics names
            if len(self.metrics_names) == 0:
                self.metrics_names = \
                    list(self.strategies[name].columns[1: -1])

    def set_plot_dimensions(self):
        """Set directives for plot sizes.

        Populates attributes: nmetrics, ncolumns, and fig_size.
        """

        # determine number of lines and columns in plot
        self.nmetrics = len(self.metrics_names)
        self.ncolumns = self.nmetrics // self.nlines + self.nmetrics % self.nlines
        self.fig_size = (10 * self.ncolumns, 7 * self.nlines)

    def plot_diagnostics(self,  output_plot_file: str, strategies_list: list):
        """
        Generate plot for all metrics in files and strategies given as input.

        Parameters
        ----------
        output_plot_file : str
            Complete path to file to store plot.
        strategies_list: list
            List of all strategies to be included in the same plot.
            Current possibibilities are:
            ['canonical', 'rand_sampling', 'unc_sampling'].
        """

        # set of all matrices to be plotted
        all_data = [self.strategies[name] for name in strategies_list]

        # set color scheme
        sns.set(rc={'axes.facecolor': 'white', 'figure.facecolor': 'white'})

        # hold axis
        axis = []

        plt.figure(figsize=self.fig_size)
        for i in range(self.nmetrics):
            ax = plt.subplot(self.nlines, self.ncolumns, i + 1)

            for j in range(len(all_data)):
                # get data
                x = all_data[j].values[:, 0]
                y = all_data[j].values[:, i + 1]

                color = self.colors[strategies_list[j]]
                marker = self.markers[strategies_list[j]]

                # plot
                ax.plot(x, y, color=color, ls=marker, lw=self.line_width,
                        label=self.labels[strategies_list[j]])

            ax.set_yticks(ax.get_yticks())
            ax.set_yticklabels([np.round(item, 2) for item in ax.get_yticks()],
                               fontsize=self.tick_label_size)
            ax.set_xticks(ax.get_xticks())
            ax.set_xticklabels([int(item)
                               for item in ax.get_xticks()],
                               fontsize=self.tick_label_size)

            ax.set_xlabel('Number of queries', fontsize=self.axis_label_size)
            ax.set_ylabel(self.metrics_names[i], fontsize=self.axis_label_size)
            ax.set_xlim(0, max(x))

            axis.append(ax)

        # handles legend
        handles, labels = axis[0].get_legend_handles_labels()
        ph = [plt.plot([], marker="", ls="")[0]]
        h = ph + handles
        rename_labels = labels
        lgd = axis[0].legend(h[1:], rename_labels, loc='upper center',
                             bbox_to_anchor=(1.025, 1.35),
                             ncol=3, fontsize=25, title='Strategy:')
        plt.setp(lgd.get_title(), fontsize='23')

        plt.subplots_adjust(left=0.075, right=0.95, top=0.875, bottom=0.075,
                            wspace=0.35, hspace=0.35)
        plt.savefig(output_plot_file)


def main():
    return None


if __name__ == '__main__':
    main()



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 10 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = ['uncertainty_sampling', 'random_sampling']

import numpy as np


def uncertainty_sampling(class_prob: np.array, test_ids: np.array,
                         queryable_ids: np.array, batch=1,
                         dump=False) -> list:
    """Search for the sample with highest uncertainty in predicted class.

    Parameters
    ----------
    class_prob: np.array
        Classification probability. One value per class per object.
    test_ids: np.array
        Set of ids for objects in the test sample.
    queryable_ids: np.array
        Set of ids for objects available for querying.
    batch: int (optional)
        Number of objects to be chosen in each batch query.
        Default is 1.
    dump: bool (optional)
        If True display on screen the shift in index and
        the difference in estimated probabilities of being Ia
        caused by constraints on the sample available for querying.

    Returns
    -------
    query_indx: list
            List of indexes identifying the objects from the test sample
            to be queried in decreasing order of importance.
    """

    # calculate distance to the decision boundary - only binary classification
    dist = abs(class_prob[:, 1] - 0.5)

    # get indexes in increasing order
    order = dist.argsort()

    # only allow objects in the query sample to be chosen
    flag = []
    for item in order:
        if test_ids[item] in queryable_ids:
            flag.append(True)
        else:
            flag.append(False)

    # arrange queryable elements in increasing order
    flag = np.array(flag)
    final_order = order[flag]

    if dump:
        print('*** Displacement caused by constraints on query****')
        print(' 0 -> ', list(order).index(final_order[0]))
        print(class_prob[order[0]], '-- > ', class_prob[final_order[0]])

    # return the index of the highest uncertain objects which are queryable
    return list(final_order)[:batch]


def random_sampling(test_ids: np.array, queryable_ids: np.array,
                    batch=1, seed=42) -> list:
    """Randomly choose an object from the test sample.

    Parameters
    ----------
    test_ids: np.array
        Set of ids for objects in the test sample.
    queryable_ids: np.array
        Set of ids for objects available for querying.
    batch: int (optional)
        Number of objects to be chosen in each batch query.
        Default is 1.
    seed: int (optional)
        Seed for random number generator. Default is 42.

    Returns
    -------
    query_indx: list
            List of indexes identifying the objects from the test sample
            to be queried.
    """

    # randomly select indexes to be queried
    np.random.seed(seed)
    indx = np.random.randint(low=0, high=len(test_ids), size=len(test_ids))

    # flag only the queryable objects
    flag = []
    for item in indx:
        if test_ids[item] in queryable_ids:
            flag.append(True)

        else:
            flag.append(False)

    flag = np.array(flag)

    # return the corresponding batch size
    return list(indx[flag])[:batch]


def main():
    return None


if __name__ == '__main__':
    main()



# Copyright 2019 snactclass software
# Author: Emille E. O. Ishida
#         Based on initial prototype developed by the CRP #4 team
#
# created on 10 August 2019
#
# Licensed GNU General Public License v3.0;
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.gnu.org/licenses/gpl-3.0.en.html
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = ['random_forest']

import numpy as np
from sklearn.ensemble import RandomForestClassifier


def random_forest(train_features:  np.array, train_labels: np.array,
                  test_features: np.array, nest=1000, seed=42):
    """Random Forest classifier.

    Parameters
    ----------
    train_features: np.array
        Training sample features.
    train_labels: np.array
        Training sample classes.
    test_features: np.array
        Test sample features.
    nest: int (optional)
        Number of estimators (trees) in the forest.
        Default is 1000.
    seed: float (optional)
        Seed for random number generator. Default is 42.

    Returns
    -------
    predictions: np.array
        Predicted classes.
    prob: np.array
        Classification probability for all objects, [pIa, pnon-Ia].
    """

    # create classifier instance
    clf = RandomForestClassifier(n_estimators=nest, random_state=seed)
    clf.fit(train_features, train_labels)                     # train
    predictions = clf.predict(test_features)                # predict
    prob = clf.predict_proba(test_features)       # get probabilities

    return predictions, prob


def main():
    return None


if __name__ == '__main__':
    main()
